// FileUtilities.cpp : Implementation of CFileUtilities
#include "stdafx.h"
#include "SecurityEnhance.h"
#include "FileUtilities.h"

/////////////////////////////////////////////////////////////////////////////
// CFileUtilities

STDMETHODIMP CFileUtilities::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IFileUtilities
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		// if (InlineIsEqualGUID(*arr[i],riid))
		// Fix for Windows 2000 RC2 SDK to compile correctly
		// second option: Put SDK ATL include dir before VC6 ATL dir
		if (::InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

#define E_FILEERROR		MAKE_HRESULT(1,FACILITY_ITF,1)
#define RET_FILE_TYPE_UNKNOWN 1
#define RET_FILE_TYPE_DISK 2
#define RET_FILE_TYPE_CHAR 3
#define RET_FILE_TYPE_PIPE 4

STDMETHODIMP CFileUtilities::GetFileType(BSTR FileName, long *FileType)
{
	USES_CONVERSION;
	*FileType = -1;

	// open the file for generic reading
	HANDLE hFile = ::CreateFile(W2A(FileName), 
						GENERIC_READ,
						FILE_SHARE_READ | FILE_SHARE_WRITE, 
						NULL, 
						OPEN_EXISTING, 
						0, 
						NULL );

	if (INVALID_HANDLE_VALUE == hFile)
	{
		TCHAR achErrFormatting[256];
		wsprintf(achErrFormatting,"The file %s could not be accessed. The Win32 error code is %ld.",W2A(FileName),::GetLastError());
		Error(achErrFormatting, 0, NULL, IID_IFileUtilities, E_FILEERROR);
		return E_FILEERROR;
	}
	else
	{
		DWORD dwFileType = ::GetFileType(hFile);
		switch (dwFileType)
		{
		case FILE_TYPE_UNKNOWN: *FileType = RET_FILE_TYPE_UNKNOWN; break;
		case FILE_TYPE_DISK: *FileType = RET_FILE_TYPE_DISK; break;
		case FILE_TYPE_CHAR: *FileType = RET_FILE_TYPE_CHAR; break;
		case FILE_TYPE_PIPE: *FileType = RET_FILE_TYPE_PIPE; break;
		default: *FileType = -1;
		}
		::CloseHandle(hFile);
	}
	return S_OK;
}

